@@ -31,7 +31,7 @@ void save_png_core(PyObject *pysurf, SDL_RWops *rw, int compress) {
     surf = PySurface_AsSurface(pysurf);
 
     /* Can't release GIL, since we're not using threaded RWops. */
-    IMG_SavePNG_RW(rw, surf, compress);
+    renpy_IMG_SavePNG_RW(rw, surf, compress);
 }
 
 /* This pixellates a 32-bit RGBA pygame surface to a destination
