@@ -39,6 +39,9 @@ ios = "RENPY_IOS" in os.environ
 # True of we're building on raspberry pi.
 raspi = "RENPY_RASPBERRY_PI" in os.environ
 
+# True of we're building with emscripten.
+emscripten = "RENPY_EMSCRIPTEN" in os.environ
+
 # Is coverage enabled?
 coverage = "RENPY_COVERAGE" in os.environ
 
@@ -58,7 +61,12 @@ cython_command = os.environ.get("RENPY_CYTHON", "cython")
 # dependencies installed in them.
 if not (android or ios):
     install = os.environ.get("RENPY_DEPS_INSTALL", "/usr")
-    install = install.split("::")
+
+    if "::" in install:
+        install = install.split("::")
+    else:
+        install = install.split(os.pathsep)
+
     install = [ os.path.abspath(i) for i in install ]
 
     if "VIRTUAL_ENV" in os.environ:
@@ -87,7 +95,7 @@ def include(header, directory=None, optional=True):
         If given, returns False rather than abandoning the process.
     """
 
-    if android or ios:
+    if android or ios or emscripten:
         return True
 
     for i in install:
@@ -126,7 +134,7 @@ def library(name, optional=False):
         rather than reporting an error.
     """
 
-    if android or ios:
+    if android or ios or emscripten:
         return True
 
     for i in install:
@@ -158,7 +166,7 @@ extensions = [ ]
 global_macros = [ ]
 
 
-def cmodule(name, source, libs=[], define_macros=[], language="c"):
+def cmodule(name, source, libs=[], define_macros=[], includes=[], language="c"):
     """
     Compiles the python module `name` from the files given in
     `source`, and the libraries in `libs`.
@@ -172,7 +180,7 @@ def cmodule(name, source, libs=[], define_macros=[], language="c"):
     extensions.append(distutils.core.Extension(
         name,
         source,
-        include_dirs=include_dirs,
+        include_dirs=include_dirs + includes,
         library_dirs=library_dirs,
         extra_compile_args=eca,
         extra_link_args=extra_link_args,
@@ -185,7 +193,7 @@ def cmodule(name, source, libs=[], define_macros=[], language="c"):
 necessary_gen = [ ]
 
 
-def cython(name, source=[], libs=[], compile_if=True, define_macros=[], pyx=None, language="c"):
+def cython(name, source=[], libs=[], includes=[], compile_if=True, define_macros=[], pyx=None, language="c"):
     """
     Compiles a cython module. This takes care of regenerating it as necessary
     when it, or any of the files it depends on, changes.
@@ -321,7 +329,7 @@ def cython(name, source=[], libs=[], compile_if=True, define_macros=[], pyx=None
         if mod_coverage:
             define_macros = define_macros + [ ("CYTHON_TRACE", "1") ]
 
-        cmodule(name, [ c_fn ] + source, libs=libs, define_macros=define_macros, language=language)
+        cmodule(name, [ c_fn ] + source, libs=libs, includes=includes, define_macros=define_macros, language=language)
 
 
 def find_unnecessary_gen():
