@@ -41,12 +41,12 @@ except ImportError:
     vc_version = 0
 
 # The tuple giving the version number.
-version_tuple = (6, 99, 14, 1, vc_version)
+version_tuple = (7, 1, 3, vc_version)
 
 # The name of this version.
-version_name = "A funny thing happened."
+version_name = "On the road again."
 
-# A string giving the version number only (7.0.1.123).
+# A string giving the version number only (8.0.1.123).
 version_only = ".".join(str(i) for i in version_tuple)
 
 # A verbose string giving the version.
@@ -70,6 +70,7 @@ macintosh = False
 linux = False
 android = False
 ios = False
+emscripten = False
 
 # Should we enable experimental features and debugging?
 experimental = "RENPY_EXPERIMENTAL" in os.environ
@@ -121,11 +122,13 @@ elif platform.mac_ver()[0]:
     macintosh = True
 elif "ANDROID_PRIVATE" in os.environ:
     android = True
+elif sys.platform == 'emscripten':
+    emscripten = True
 else:
     linux = True
 
 # A flag that's true if we're on a smartphone or tablet-like platform.
-mobile = android or ios
+mobile = android or ios or emscripten
 
 # A flag that's set to true if the game directory is bundled inside a mac app.
 macapp = False
@@ -534,6 +537,8 @@ def post_import():
     for k, v in renpy.defaultstore.__dict__.iteritems():
         renpy.store.__dict__.setdefault(k, v)
 
+    renpy.store.eval = renpy.defaultstore.eval
+
     # Import everything into renpy.exports, provided it isn't
     # already there.
     for k, v in globals().iteritems():
