@@ -26,6 +26,8 @@
 # When updating this file, consider if lint.py or warp.py also need
 # updating.
 
+from __future__ import print_function
+
 import renpy.display
 import renpy.test
 
@@ -191,6 +193,24 @@ class ArgumentInfo(object):
 
         return tuple(args), kwargs
 
+    def get_code(self):
+
+        l = [ ]
+
+        for keyword, expression in self.arguments:
+            if keyword is not None:
+                l.append("{}={}".format(keyword, expression))
+            else:
+                l.append(expression)
+
+        if self.extrapos is not None:
+            l.append("*" + self.extrapos)
+
+        if self.extrakw is not None:
+            l.append("**" + self.extrakw)
+
+        return "(" + ", ".join(l) + ")"
+
 
 def __newobj__(cls, *args):
     return cls.__new__(cls, *args)
@@ -311,7 +331,10 @@ class Scry(object):
         if self._next is None:
             return None
         else:
-            return self._next.scry()
+            try:
+                return self._next.scry()
+            except:
+                return None
 
 
 class Node(object):
@@ -479,6 +502,15 @@ class Node(object):
         # Does nothing by default.
         return
 
+    warp = False
+
+    def can_warp(self):
+        """
+        Returns true if this should be run while warping, False otherwise.
+        """
+
+        return self.warp
+
 
 def say_menu_with(expression, callback):
     """
@@ -601,6 +633,9 @@ class Say(Node):
             rv.append("with")
             rv.append(self.with_)
 
+        if self.arguments:
+            rv.append(self.arguments.get_code())
+
         return " ".join(rv)
 
     def execute(self):
@@ -674,13 +709,7 @@ class Say(Node):
     def scry(self):
         rv = Node.scry(self)
 
-        if self.who is not None:
-            if self.who_fast:
-                who = getattr(renpy.store, self.who)
-            else:
-                who = renpy.python.py_eval(self.who)
-        else:
-            who = None
+        who = eval_who(self.who, self.who_fast)
 
         if self.interact:
             renpy.exports.scry_say(who, rv)
@@ -738,7 +767,6 @@ class Label(Node):
     rollback = "force"
 
     translation_relevant = True
-
     __slots__ = [
         'name',
         'parameters',
@@ -831,7 +859,12 @@ class Python(Node):
         super(Python, self).__init__(loc)
 
         self.hide = hide
-        self.code = PyCode(python_code, loc=loc, mode='exec')
+
+        if hide:
+            self.code = PyCode(python_code, loc=loc, mode='hide')
+        else:
+            self.code = PyCode(python_code, loc=loc, mode='exec')
+
         self.store = store
 
     def diff_info(self):
@@ -882,7 +915,12 @@ class EarlyPython(Node):
         super(EarlyPython, self).__init__(loc)
 
         self.hide = hide
-        self.code = PyCode(python_code, loc=loc, mode='exec')
+
+        if hide:
+            self.code = PyCode(python_code, loc=loc, mode='hide')
+        else:
+            self.code = PyCode(python_code, loc=loc, mode='exec')
+
         self.store = store
 
     def diff_info(self):
@@ -1089,6 +1127,8 @@ class Show(Node):
         'atl',
         ]
 
+    warp = True
+
     def __init__(self, loc, imspec, atl=None):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1120,6 +1160,8 @@ class Show(Node):
 
 class ShowLayer(Node):
 
+    warp = True
+
     __slots__ = [
         'layer',
         'at_list',
@@ -1164,6 +1206,8 @@ class Scene(Node):
         'atl',
         ]
 
+    warp = True
+
     def __init__(self, loc, imgspec, layer, atl=None):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1214,6 +1258,8 @@ class Hide(Node):
         'imspec',
         ]
 
+    warp = True
+
     def __init__(self, loc, imgspec):
         """
         @param imspec: A triple consisting of an image name (itself a
@@ -1362,6 +1408,9 @@ class Call(Node):
             args, kwargs = self.arguments.evaluate()
             renpy.store._args = args
             renpy.store._kwargs = kwargs
+        else:
+            renpy.store._args = None
+            renpy.store._kwargs = None
 
     def predict(self):
 
@@ -1439,14 +1488,21 @@ class Menu(Node):
         'items',
         'set',
         'with_',
+        'has_caption',
         ]
 
-    def __init__(self, loc, items, set, with_):  # @ReservedAssignment
+    def __new__(cls, *args, **kwargs):
+        self = Node.__new__(cls)
+        self.has_caption = False
+        return self
+
+    def __init__(self, loc, items, set, with_, has_caption):  # @ReservedAssignment
         super(Menu, self).__init__(loc)
 
         self.items = items
         self.set = set
         self.with_ = with_
+        self.has_caption = has_caption
 
     def diff_info(self):
         return (Menu,)
@@ -1478,7 +1534,11 @@ class Menu(Node):
     def execute(self):
 
         next_node(self.next)
-        statement_name("menu")
+
+        if self.has_caption:
+            statement_name("menu-with-caption")
+        else:
+            statement_name("menu")
 
         choices = [ ]
         narration = [ ]
@@ -1838,6 +1898,13 @@ class UserStatement(Node):
     def get_code(self, dialogue_filter=None):
         return self.line
 
+    def can_warp(self):
+
+        if self.call("warp"):
+            return True
+
+        return False
+
 
 def create_store(name):
     if name not in renpy.config.special_namespaces:
@@ -2001,7 +2068,10 @@ class Default(Node):
             d.ever_been_changed.add("_defaults_set")
 
         if self.varname not in defaults_set:
-            d[self.varname] = renpy.python.py_eval_bytecode(self.code.bytecode)
+
+            if start or (self.varname not in d.ever_been_changed):
+                d[self.varname] = renpy.python.py_eval_bytecode(self.code.bytecode)
+
             d.ever_been_changed.add(self.varname)
 
             defaults_set.add(self.varname)
@@ -2068,15 +2138,17 @@ class Translate(Node):
 
     __slots__ = [
         "identifier",
+        "alternate",
         "language",
         "block",
         "after",
         ]
 
-    def __init__(self, loc, identifier, language, block):
+    def __init__(self, loc, identifier, language, block, alternate=None):
         super(Translate, self).__init__(loc)
 
         self.identifier = identifier
+        self.alternate = alternate
         self.language = language
         self.block = block
 
@@ -2101,6 +2173,9 @@ class Translate(Node):
         if self.after is old:
             self.after = new
 
+    def lookup(self):
+        return renpy.game.script.translator.lookup_translate(self.identifier, getattr(self, "alternate", None))
+
     def execute(self):
 
         statement_name("translate")
@@ -2114,18 +2189,18 @@ class Translate(Node):
             renpy.game.seen_translates_count += 1
             renpy.game.new_translates_count += 1
 
-        next_node(renpy.game.script.translator.lookup_translate(self.identifier))
+        next_node(self.lookup())
 
         renpy.game.context().translate_identifier = self.identifier
-        renpy.game.context().translate_block_language = self.language
+        renpy.game.context().alternate_translate_identifier = getattr(self, "alternate", None)
 
     def predict(self):
-        node = renpy.game.script.translator.lookup_translate(self.identifier)
+        node = self.lookup()
         return [ node ]
 
     def scry(self):
         rv = Scry()
-        rv._next = renpy.game.script.translator.lookup_translate(self.identifier)
+        rv._next = self.lookup()
         return rv
 
     def get_children(self, f):
@@ -2157,7 +2232,7 @@ class EndTranslate(Node):
         statement_name("end translate")
 
         renpy.game.context().translate_identifier = None
-        renpy.game.context().translate_block_language = None
+        renpy.game.context().alternate_translate_identifier = None
 
 
 class TranslateString(Node):
