@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,6 +21,8 @@
 
 # The public API for music in games.
 
+from __future__ import print_function
+
 import renpy.audio
 
 from renpy.audio.audio import get_channel, get_serial
@@ -62,7 +64,8 @@ def play(filenames, channel="music", loop=None, fadeout=None, synchro_start=Fals
         first loop only.
 
     `tight`
-        If this is True, then fadeouts will span into the next-queued sound.
+        If this is True, then fadeouts will span into the next-queued sound. If
+        None, this is true when loop is True, and false otherwise.
 
     `if_changed`
         If this is True, and the music file is currently playing,
@@ -82,45 +85,56 @@ def play(filenames, channel="music", loop=None, fadeout=None, synchro_start=Fals
     if isinstance(filenames, basestring):
         filenames = [ filenames ]
 
-    try:
-        c = get_channel(channel)
-        ctx = c.context
+    with renpy.audio.audio.lock:
 
-        if loop is None:
-            loop = c.default_loop
+        try:
+            c = get_channel(channel)
+            ctx = c.context
 
-        loop_is_filenames = (c.loop == filenames)
+            if loop is None:
+                loop = c.default_loop
 
-        c.dequeue()
+            if (tight is None) and renpy.config.tight_loop_default:
+                tight = loop
 
-        if fadeout is None:
-            fadeout = renpy.config.fade_music
+            loop_is_filenames = (c.loop == filenames)
 
-        if if_changed and c.get_playing() in filenames:
-            fadein = 0
-            loop_only = loop_is_filenames
-        else:
-            c.fadeout(fadeout)
-            loop_only = False
+            c.dequeue()
 
-        c.enqueue(filenames, loop=loop, synchro_start=synchro_start, fadein=fadein, tight=tight, loop_only=loop_only)
+            if fadeout is None:
+                fadeout = renpy.config.fade_music
 
-        t = get_serial()
-        ctx.last_changed = t
-        c.last_changed = t
+            if if_changed and c.get_playing() in filenames:
+                fadein = 0
+                loop_only = loop_is_filenames
+            else:
+                c.fadeout(fadeout)
+                loop_only = False
 
-        if loop:
-            ctx.last_filenames = filenames
-            ctx.last_tight = tight
-        else:
-            ctx.last_filenames = [ ]
-            ctx.last_tight = False
+            if renpy.config.skip_sounds and renpy.config.skipping and (not loop):
+                enqueue = False
+            else:
+                enqueue = True
 
-        ctx.pause = False
+            if enqueue:
+                c.enqueue(filenames, loop=loop, synchro_start=synchro_start, fadein=fadein, tight=tight, loop_only=loop_only)
 
-    except:
-        if renpy.config.debug_sound:
-            raise
+            t = get_serial()
+            ctx.last_changed = t
+            c.last_changed = t
+
+            if loop:
+                ctx.last_filenames = filenames
+                ctx.last_tight = tight
+            else:
+                ctx.last_filenames = [ ]
+                ctx.last_tight = False
+
+            ctx.pause = False
+
+        except:
+            if renpy.config.debug_sound:
+                raise
 
 
 def queue(filenames, channel="music", loop=None, clear_queue=True, fadein=0, tight=None):
@@ -150,7 +164,8 @@ def queue(filenames, channel="music", loop=None, clear_queue=True, fadein=0, tig
         first loop only.
 
     `tight`
-        If this is True, then fadeouts will span into the next-queued sound.
+        If this is True, then fadeouts will span into the next-queued sound. If
+        None, this is true when loop is True, and false otherwise.
 
     This clears the pause flag for `channel`.
     """
@@ -165,35 +180,49 @@ def queue(filenames, channel="music", loop=None, clear_queue=True, fadein=0, tig
     if isinstance(filenames, basestring):
         filenames = [ filenames ]
 
-    try:
+    if renpy.config.skipping == "fast":
+        stop(channel)
 
-        c = get_channel(channel)
-        ctx = c.context
+    with renpy.audio.audio.lock:
 
-        if loop is None:
-            loop = c.default_loop
+        try:
 
-        if clear_queue:
-            c.dequeue(True)
+            c = get_channel(channel)
+            ctx = c.context
 
-        c.enqueue(filenames, loop=loop, fadein=fadein, tight=tight)
+            if loop is None:
+                loop = c.default_loop
 
-        t = get_serial()
-        ctx.last_changed = t
-        c.last_changed = t
+            if (tight is None) and renpy.config.tight_loop_default:
+                tight = loop
 
-        if loop:
-            ctx.last_filenames = filenames
-            ctx.last_tight = tight
-        else:
-            ctx.last_filenames = [ ]
-            ctx.last_tight = False
+            if clear_queue:
+                c.dequeue(True)
 
-        ctx.pause = False
+            if renpy.config.skip_sounds and renpy.config.skipping and (not loop):
+                enqueue = False
+            else:
+                enqueue = True
 
-    except:
-        if renpy.config.debug_sound:
-            raise
+            if enqueue:
+                c.enqueue(filenames, loop=loop, fadein=fadein, tight=tight)
+
+            t = get_serial()
+            ctx.last_changed = t
+            c.last_changed = t
+
+            if loop:
+                ctx.last_filenames = filenames
+                ctx.last_tight = tight
+            else:
+                ctx.last_filenames = [ ]
+                ctx.last_tight = False
+
+            ctx.pause = False
+
+        except:
+            if renpy.config.debug_sound:
+                raise
 
 
 def playable(filename, channel="music"):
@@ -234,24 +263,26 @@ def stop(channel="music", fadeout=None):
     if renpy.game.context().init_phase:
         return
 
-    try:
-        c = get_channel(channel)
-        ctx = c.context
+    with renpy.audio.audio.lock:
 
-        if fadeout is None:
-            fadeout = renpy.config.fade_music
+        try:
+            c = get_channel(channel)
+            ctx = c.context
 
-        c.fadeout(fadeout)
+            if fadeout is None:
+                fadeout = renpy.config.fade_music
 
-        t = get_serial()
-        ctx.last_changed = t
-        c.last_changed = t
-        ctx.last_filenames = [ ]
-        ctx.last_tight = False
+            c.fadeout(fadeout)
 
-    except:
-        if renpy.config.debug_sound:
-            raise
+            t = get_serial()
+            ctx.last_changed = t
+            c.last_changed = t
+            ctx.last_filenames = [ ]
+            ctx.last_tight = False
+
+        except:
+            if renpy.config.debug_sound:
+                raise
 
 
 def set_music(channel, flag, default=False):
@@ -310,7 +341,7 @@ def get_pos(channel="music"):
     seconds. Returns None if no audio is playing on `channel`.
 
     As this may return None before a channel starts playing, or if the audio
-    channel involved has been muted, code that calls this function should
+    channel involved has been muted, callers of this function should
     always handle a None value.
     """
 
