@@ -22,6 +22,8 @@
 # TODO: Use overlap (rather than simple pointer location) to determine
 # drag and drop.
 
+from __future__ import print_function
+
 import renpy.display
 from renpy.display.render import render, Render, redraw
 from renpy.display.core import absolute
@@ -45,10 +47,11 @@ def default_drag_group():
 
     return rv
 
-
 def default_drag_joined(drag):
     return [ (drag, 0, 0) ]
 
+def default_drop_allowable(drop, drags):
+    return True
 
 class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     """
@@ -105,6 +108,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         If true, this Drag is raised to the top when it is dragged. If
         it is joined to other Drags, all joined drags are raised.
 
+    `activated`
+        A callback (or list of callbacks) that is called when the mouse
+        is pressed down on the drag. It is called with one argument, a
+        a list of Drags that are being dragged. The return value of this
+        callback is ignored.
+
     `dragged`
         A callback (or list of callbacks) that is called when the Drag
         has been dragged. It is called with two arguments. The first is
@@ -130,6 +139,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         and clicked.  If the callback returns a value other than None,
         that value is returned as the result of the interaction.
 
+    `alternate`
+        An action that is run when the Drag is right-clicked (on the
+        desktop) or long-pressed without moving (on mobile). It may
+        be necessary to increase :var:`config.longpress_duration` if
+        this triggers to early on mobile platforms.
+
     `drag_handle`
         A (x, y, width, height) tuple, giving the position of the drag
         handle within the child. In this tuple, integers are considered
@@ -150,10 +165,16 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         way to get them back on the screen.
 
     `mouse_drop`
-        If true, the drag is dropped on the first droppable under the cursor. 
-        If false, the default, the drag is dropped onto the droppable with 
+        If true, the drag is dropped on the first droppable under the cursor.
+        If false, the default, the drag is dropped onto the droppable with
         the largest degree of overlap.
 
+    `drop_allowable`
+        A callback that is called to determine whether this drop allow
+        the current drags dropped onto. It is called with two arguments.
+        The first is the Drag which determines its sensitivity.
+        The second is a list of Drags that are being dragged.
+
     Except for `d`, all of the parameters are available as fields (with
     the same name) on the Drag object. In addition, after the drag has
     been rendered, the following fields become available:
@@ -163,13 +184,18 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
     `w`, `h`
          The width and height of the Drag's child, in pixels.
-        """
+    """
 
     focusable = True
 
     drag_group = None
     old_position = None
     drag_offscreen = False
+    activated = None
+    alternate = None
+
+    # The time a click started, or None if a click is not in progress.
+    click_time = None
 
     def __init__(self,
                  d=None,
@@ -179,6 +205,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                  drag_raise=True,
                  dragged=None,
                  dropped=None,
+                 drop_allowable=default_drop_allowable,
                  drag_handle=(0.0, 0.0, 1.0, 1.0),
                  drag_joined=default_drag_joined,
                  clicked=None,
@@ -187,6 +214,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                  replaces=None,
                  drag_offscreen=False,
                  mouse_drop=False,
+                 activated=None,
+                 alternate=None,
                  style="drag",
                  **properties):
 
@@ -198,11 +227,14 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         self.drag_raise = drag_raise
         self.dragged = dragged
         self.dropped = dropped
+        self.drop_allowable = drop_allowable
         self.drag_handle = drag_handle
         self.drag_joined = drag_joined
         self.clicked = clicked
         self.hovered = hovered
         self.unhovered = unhovered
+        self.activated = activated
+        self.alternate = alternate
         self.drag_offscreen = drag_offscreen
         # if mouse_drop_check is True (default False), the drop will not
         #  use default major overlap between droppables but instead
@@ -283,6 +315,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             self.drag_moved = replaces.drag_moved
             self.last_drop = replaces.last_drop
             self.mouse_drop = replaces.mouse_drop
+            self.click_time = replaces.click_time
 
         if d is not None:
             self.add(d)
@@ -515,6 +548,10 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
         grabbed = (renpy.display.focus.get_grab() is self)
 
+        if (self.alternate is not None) and renpy.display.touch and map_event(ev, "drag_activate"):
+            self.click_time = st
+            renpy.game.interface.timeout(renpy.config.longpress_duration)
+
         if grabbed:
             joined_offsets = self.drag_joined(self)
             joined = [ i[0] for i in joined_offsets ]
@@ -529,6 +566,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             renpy.display.focus.set_grab(self)
 
+            run(joined[0].activated, joined)
+
             self.grab_x = x
             self.grab_y = y
 
@@ -546,9 +585,30 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             grabbed = True
 
+        elif (self.alternate is not None) and map_event(ev, "button_alternate"):
+            rv = run(self.alternate)
+            if rv is not None:
+                return rv
+
+            raise renpy.display.core.IgnoreEvent()
+
+        if ((self.alternate is not None) and
+            renpy.display.touch and
+            (self.click_time is not None) and
+                ((st - self.click_time) > renpy.config.longpress_duration)):
+
+            self.click_time = None
+
+            rv = run(self.alternate)
+            if rv is not None:
+                return rv
+
         # Handle clicking on droppables.
         if not grabbed:
             if self.clicked is not None and map_event(ev, "drag_deactivate"):
+
+                self.click_time = None
+
                 rv = run(self.clicked)
                 if rv is not None:
                     return rv
@@ -564,6 +624,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
             if not self.drag_moved and (self.start_x != par_x or self.start_y != par_y):
                 self.drag_moved = True
+                self.click_time = None
 
                 # We may not be in the drag_joined group.
                 self.set_style_prefix("idle_", True)
@@ -603,9 +664,9 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
         if (self.drag_group is not None) and self.drag_moved:
             if self.mouse_drop:
-               drop = self.drag_group.get_drop_at(joined, par_x, par_y)
+                drop = self.drag_group.get_drop_at(joined, par_x, par_y)
             else:
-               drop = self.drag_group.get_best_drop(joined)
+                drop = self.drag_group.get_best_drop(joined)
         else:
             drop = None
 
@@ -620,6 +681,9 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             self.last_drop = drop
 
         if map_event(ev, 'drag_deactivate'):
+
+            self.click_time = None
+
             renpy.display.focus.set_grab(None)
 
             if drop is not None:
@@ -685,6 +749,11 @@ class DragGroup(renpy.display.layout.MultiBox):
 
     All positional parameters to the DragGroup constructor should be
     Drags, that are added to the DragGroup.
+
+
+    `min_overlap`
+        An integer which means the minimum number of pixels at the
+        overlap so that drop will be allow.
     """
 
     _list_type = renpy.python.RevertableList
@@ -698,6 +767,9 @@ class DragGroup(renpy.display.layout.MultiBox):
 
         replaces = properties.pop("replaces", None)
 
+        min_overlap = properties.pop("min_overlap", 0)
+        self.min_overlap = min_overlap
+
         super(DragGroup, self).__init__(**properties)
 
         if replaces is not None:
@@ -818,7 +890,11 @@ class DragGroup(renpy.display.layout.MultiBox):
 
                 overlap = rect_overlap_area(r1, r2)
 
-                if overlap >= max_overlap:
+                if (
+                    overlap >= max_overlap and
+                    overlap >= self.min_overlap and
+                    c.drop_allowable(c, joined)
+                ):
                     rv = c
                     max_overlap = overlap
 
@@ -835,17 +911,20 @@ class DragGroup(renpy.display.layout.MultiBox):
         joined_set = set(joined)
         for c in self.children:
             if c in joined_set:
-               continue
+                continue
 
             if not c.droppable:
-               continue
+                continue
 
             if c.x is None:
-               continue
+                continue
 
-            if (x >= c.x and y >= c.y and
-                x < (c.x+c.w) and y < (c.y+c.h)):
-               return c
+            if (
+                x >= c.x and y >= c.y and
+                x < (c.x + c.w) and y < (c.y + c.h) and
+                c.drop_allowable(c, joined)
+            ):
+                return c
 
     def get_children(self):
         """
