@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,6 +21,8 @@
 
 # This code supports sprite and particle animation.
 
+from __future__ import print_function
+
 from renpy.display.render import render, BLIT
 
 import renpy.display
@@ -103,6 +105,7 @@ class Sprite(renpy.object.Object):
 
             if d._duplicatable:
                 sc.child_copy = d._duplicate(None)
+                sc.child_copy._unique()
             else:
                 sc.child_copy = d
 
