@@ -40,6 +40,7 @@ def get_null():
 
     if null is None:
         null = renpy.display.layout.Null()
+        renpy.display.motion.null = null
 
     return null
 
@@ -691,6 +692,9 @@ class Transform(Container):
 
         self.state.take_state(t.state)
 
+        if isinstance(self.child, Transform) and isinstance(t.child, Transform):
+            self.child.take_state(t.child)
+
         if (self.child is None) and (t.child is not None):
             self.add(t.child)
             self.child_st_base = t.child_st_base
@@ -979,6 +983,9 @@ class Transform(Container):
 
     def _duplicate(self, args):
 
+        if args and args.args:
+            args.extraneous()
+
         if not self._duplicatable:
             return self
 
