@@ -72,6 +72,7 @@ from renpy.display.predict import screen as predict_screen
 
 from renpy.display.image import image_exists, image_exists as has_image, list_images
 from renpy.display.image import get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
+from renpy.display.image import get_registered_image
 
 from renpy.display.im import load_surface, load_image
 
@@ -86,7 +87,10 @@ from renpy.python import rng as random
 from renpy.atl import atl_warper
 from renpy.easy import predict, displayable, split_properties
 from renpy.parser import unelide_filename, get_parse_errors
+
 from renpy.translation import change_language, known_languages
+from renpy.translation.generation import generic_filter as transform_text
+
 
 from renpy.persistent import register_persistent
 
@@ -157,6 +161,7 @@ def public_api():
     displayable, split_properties
     unelide_filename, get_parse_errors
     change_language, known_languages
+    transform_text
     language_tailor
     register_persistent
     register_statement
@@ -165,6 +170,7 @@ def public_api():
     const, pure, not_const
     image_exists, has_image, list_images
     get_available_image_tags, get_available_image_attributes, check_image_attributes, get_ordered_image_attributes
+    get_registered_image
     load_image, load_surface
     profile_memory, diff_memory, profile_rollback
     TEXT_TAG
@@ -232,6 +238,9 @@ def can_rollback():
     Returns true if we can rollback.
     """
 
+    if not renpy.config.rollback_enabled:
+        return False
+
     return renpy.game.log.can_rollback()
 
 
@@ -266,15 +275,16 @@ def checkpoint(data=None, keep_rollback=None):
     renpy.game.log.checkpoint(data, keep_rollback=keep_rollback, hard=renpy.store._rollback)
 
 
-def block_rollback():
+def block_rollback(purge=False):
     """
     :doc: blockrollback
+    :args: ()
 
     Prevents the game from rolling back to before the current
     statement.
     """
 
-    renpy.game.log.block()
+    renpy.game.log.block(purge=purge)
 
 
 def suspend_rollback(flag):
@@ -331,7 +341,7 @@ def image(name, d):
     """
     :doc: se_images
 
-    Defines an image. This function is the python equivalent of the
+    Defines an image. This function is the Python equivalent of the
     image statement.
 
     `name`
@@ -456,7 +466,7 @@ def showing(name, layer='master'):
     :doc: image_func
 
     Returns true if an image with the same tag as `name` is showing on
-    `layer`
+    `layer`.
 
     `image`
         May be a string giving the image name or a tuple giving each
@@ -476,22 +486,38 @@ def showing(name, layer='master'):
     return renpy.game.context().images.showing(layer, name)
 
 
-def get_showing_tags(layer='master'):
+def get_showing_tags(layer='master', sort=False):
     """
     :doc: image_func
 
-    Returns the set of image tags that are currently being shown on `layer`
+    Returns the set of image tags that are currently being shown on `layer`. If
+    sort is true, returns a list of the tags from back to front.
     """
 
+    if sort:
+        return scene_lists().get_sorted_tags(layer)
+
     return renpy.game.context().images.get_showing_tags(layer)
 
 
+def get_hidden_tags(layer='master'):
+    """
+    :doc: image_func
+
+    Returns the set of image tags on `layer` that are currently hidden, but
+    still have attribute information associated with them.
+    """
+
+    return renpy.game.context().images.get_hidden_tags(layer)
+
+
 def get_attributes(tag, layer=None):
     """
     :doc: image_func
 
-    Return a tuple giving the image attributes for the image with `tag`. If
-    the image is now showing, returns None.
+    Return a tuple giving the image attributes for the image `tag`. If
+    the image tag has not had any attributes associated since the last
+    time it was hidden, returns None.
 
     `layer`
         The layer to check. If None, uses the default layer for `tag`.
@@ -687,7 +713,7 @@ def hide(name, layer=None):
     """
     :doc: se_images
 
-    Hides an image from a layer. The python equivalent of the hide statement.
+    Hides an image from a layer. The Python equivalent of the hide statement.
 
     `name`
          The name of the image to hide. Only the image tag is used, and
@@ -847,9 +873,29 @@ def menu(items, set_expr):
 
     # Filter the list of items to only include ones for which the
     # condition is true.
-    items = [ (substitute(label), value)
-              for label, condition, value in items
-              if renpy.python.py_eval(condition) ]
+
+    if renpy.config.menu_actions:
+
+        location=renpy.game.context().current
+
+        new_items = [ ]
+
+        for label, condition, value in items:
+            label = substitute(label)
+            condition = renpy.python.py_eval(condition)
+
+            if (not renpy.config.menu_include_disabled) and (not condition):
+                continue
+
+            new_items.append((label, renpy.ui.ChoiceReturn(label, value, location, sensitive=condition)))
+
+        items = new_items
+
+    else:
+
+        items = [ (substitute(label), value)
+                  for label, condition, value in items
+                  if renpy.python.py_eval(condition) ]
 
     # Filter the list of items on the set_expr:
     if set_expr:
@@ -878,6 +924,7 @@ def menu(items, set_expr):
                     set.append(label)
                 except AttributeError:
                     set.add(label)
+
     return rv
 
 
@@ -947,6 +994,7 @@ def display_menu(items,
                  **kwargs):
     """
     :doc: se_menu
+    :name: renpy.display_menu
     :args: (items, interact=True, screen="choice")
 
     This displays a menu to the user. `items` should be a list of 2-item tuples.
@@ -1008,7 +1056,11 @@ def display_menu(items,
             if not label:
                 value = None
 
-            if value is not None:
+            if isinstance(value, renpy.ui.ChoiceReturn):
+                action = value
+                chosen = action.get_chosen()
+
+            elif value is not None:
                 action = renpy.ui.ChoiceReturn(label, value, location)
                 chosen = action.get_chosen()
             else:
@@ -1041,7 +1093,8 @@ def display_menu(items,
                       choice_chosen_button_style=choice_chosen_button_style,
                       **kwargs)
 
-    renpy.exports.shown_window()
+    if renpy.config.menu_showed_window:
+        renpy.exports.shown_window()
 
     # Log the chosen choice.
     for label, val in items:
@@ -1381,7 +1434,7 @@ def with_statement(trans, always=False, paired=None, clear=True):
     :name: renpy.with_statement
     :args: (trans, always=False)
 
-    Causes a transition to occur. This is the python equivalent of the
+    Causes a transition to occur. This is the Python equivalent of the
     with statement.
 
     `trans`
@@ -1406,15 +1459,35 @@ def with_statement(trans, always=False, paired=None, clear=True):
 
     renpy.exports.mode('with')
 
+    if isinstance(paired, dict):
+        paired = paired.get(None, None)
+
+        if (trans is None) and (paired is None):
+            return
+
+    if isinstance(trans, dict):
+
+        for k, v in trans.items():
+            if k is None:
+                continue
+
+            renpy.exports.transition(v, layer=k)
+
+        if None not in trans:
+            return
+
+        trans = trans[None]
+
     return renpy.game.interface.do_with(trans, paired, clear=clear)
 
 
 globals()["with"] = with_statement
 
 
-def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True):
+def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True, current_label=None):
     """
     :doc: rollback
+    :args: (force=False, checkpoints=1, defer=False, greedy=True, label=None, abnormal=True)
 
     Rolls the state of the game back to the last checkpoint.
 
@@ -1461,7 +1534,7 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None, a
 
     renpy.config.skipping = None
     renpy.game.log.complete()
-    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True), abnormal=abnormal)
+    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True), abnormal=abnormal, current_label=current_label)
 
 
 def toggle_fullscreen():
@@ -1564,6 +1637,10 @@ def reload_script():
 
     s = get_screen("menu")
 
+    session.pop("_reload_screen", None)
+    session.pop("_reload_screen_args", None)
+    session.pop("_reload_screen_kwargs", None)
+
     if not renpy.store.main_menu:
 
         if s is not None:
@@ -1583,7 +1660,7 @@ def reload_script():
         utter_restart()
 
 
-def quit(relaunch=False, status=0):  # @ReservedAssignment
+def quit(relaunch=False, status=0, save=False):  # @ReservedAssignment
     """
     :doc: other
 
@@ -1595,8 +1672,18 @@ def quit(relaunch=False, status=0):  # @ReservedAssignment
     `status`
         The status code Ren'Py will return to the operating system.
         Generally, 0 is success, and positive integers are failure.
+
+    `save`
+        If true, the game is saved in :var:`_quit_slot` before Ren'Py
+        terminates.
     """
 
+    if save and (renpy.store._quit_slot is not None):
+        renpy.loadsave.save(renpy.store._quit_slot, getattr(renpy.store, "save_name", ""))
+
+    if has_label("quit"):
+        call_in_new_context("quit")
+
     raise renpy.game.QuitException(relaunch=relaunch, status=status)
 
 
@@ -1696,6 +1783,8 @@ except:
         platform = "Android"
     elif renpy.ios:
         platform = "iOS"
+    elif sys.platform == "emscripten":
+        platform = "emscripten"
     else:
         platform = "Unknown"
 
@@ -1716,7 +1805,12 @@ def transition(trans, layer=None, always=False, force=False):
         transition is always run.
     """
 
-    if not always and not renpy.game.preferences.transitions:
+    if isinstance(trans, dict):
+        for layer, t in trans.items():
+            transition(t, layer=layer, always=always, force=force)
+        return
+
+    if (not always) and not renpy.game.preferences.transitions:
         trans = None
 
     renpy.game.interface.set_transition(trans, layer, force=force)
@@ -2035,7 +2129,7 @@ def file(fn):  # @ReservedAssignment
     accessed using Ren'Py's standard search method, and may reside in an RPA archive.
     or as an Android asset.
 
-    The object supports a wide subset of the fields and methods found on python's
+    The object supports a wide subset of the fields and methods found on Python's
     standard file object, opened in binary mode. (Basically, all of the methods that
     are sensible for a read-only file.)
     """
@@ -2100,7 +2194,7 @@ def show_layer_at(at_list, layer='master', reset=True):
     :doc: se_images
     :name: renpy.show_layer_at
 
-    The python equivalent of the ``show layer`` `layer` ``at`` `at_list`
+    The Python equivalent of the ``show layer`` `layer` ``at`` `at_list`
     statement.
 
     `reset`
@@ -2387,7 +2481,7 @@ def game_menu(screen=None):
     if screen is None:
         call_in_new_context("_game_menu")
     else:
-        call_in_new_context("_game_menu", screen)
+        call_in_new_context("_game_menu", _game_menu_screen=screen)
 
 
 def shown_window():
@@ -2427,13 +2521,14 @@ def get_placement(d):
     This returns an object with the following fields, each corresponding to a style
     property:
 
-        * xpos
-        * xanchor
-        * xoffset
-        * ypos
-        * yanchor
-        * yoffset
-        * subpixel
+    * xpos
+    * xanchor
+    * xoffset
+    * ypos
+    * yanchor
+    * yoffset
+    * subpixel
+
     """
     p = d.get_placement()
 
@@ -2647,6 +2742,11 @@ def call_screen(_screen_name, *args, **kwargs):
 
     renpy.exports.mode('screen')
 
+    with_none = renpy.config.implicit_with_none
+
+    if "_with_none" in kwargs:
+        with_none = kwargs.pop("_with_none")
+
     show_screen(_screen_name, _transient=True, *args, **kwargs)
 
     roll_forward = renpy.exports.roll_forward_info()
@@ -2658,11 +2758,6 @@ def call_screen(_screen_name, *args, **kwargs):
 
     renpy.exports.checkpoint(rv)
 
-    with_none = renpy.config.implicit_with_none
-
-    if "_with_none" in kwargs:
-        with_none = kwargs.pop("_with_none")
-
     if with_none:
         renpy.game.interface.do_with(None, None)
 
@@ -2808,7 +2903,7 @@ def variant(name):
 
     Returns true if a `name` is a screen variant that can be chosen
     by Ren'Py. See :ref:`screen-variants` for more details. This function
-    can be used as the condition in a python if statement to set up the
+    can be used as the condition in a Python if statement to set up the
     appropriate styles for the selected screen variant.
 
     `name` can also be a list of variants, in which case this function
@@ -2853,7 +2948,7 @@ def get_say_attributes():
     return renpy.game.context().say_attributes
 
 
-def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer='master'):
+def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer=None):
     """
     :doc: other
 
@@ -3509,3 +3604,19 @@ def get_on_battery():
     else:
         old_battery = False
         return False
+
+
+def get_say_image_tag():
+    """
+    :doc: image_func
+
+    Returns the tag corresponding to the currently speaking character (the
+    `image` argument given to that character). Returns None if no character
+    is speaking or the current speaking character does not have a corresponding
+    image tag.
+    """
+
+    if renpy.store._side_image_attributes is None:
+        return None
+
+    return renpy.store._side_image_attributes[0]
