@@ -558,7 +558,7 @@ class Lexer(object):
     sub-lexers to lex sub-blocks.
     """
 
-    def __init__(self, block, init=False, init_offset=0, global_label=None):
+    def __init__(self, block, init=False, init_offset=0, global_label=None, monologue_delimiter="\n\n"):
 
         # Are we underneath an init block?
         self.init = init
@@ -582,6 +582,8 @@ class Lexer(object):
         self.word_cache_newpos = -1
         self.word_cache = ""
 
+        self.monologue_delimiter = monologue_delimiter
+
     def advance(self):
         """
         Advances this lexer to the next line in the block. The lexer
@@ -716,7 +718,7 @@ class Lexer(object):
 
         init = self.init or init
 
-        return Lexer(self.subblock, init=init, init_offset=self.init_offset, global_label=self.global_label)
+        return Lexer(self.subblock, init=init, init_offset=self.init_offset, global_label=self.global_label, monologue_delimiter=self.monologue_delimiter)
 
     def string(self):
         """
@@ -775,6 +777,75 @@ class Lexer(object):
 
         return s
 
+    def triple_string(self):
+        """
+        Lexes a triple quoted string, intended for use with monologue mode.
+        This is about the same as the double-quoted strings, except that
+        runs of whitespace with multiple newlines are turned into a single
+        newline.
+        """
+
+        s = self.match(r'r?"""([^\\"]|\\.)*"""')
+
+        if s is None:
+            s = self.match(r"r?'''([^\\']|\\.)*'''")
+
+        if s is None:
+            s = self.match(r"r?```([^\\`]|\\.)*```")
+
+        if s is None:
+            return None
+
+        if s[0] == 'r':
+            raw = True
+            s = s[1:]
+        else:
+            raw = False
+
+        # Strip off delimiters.
+        s = s[3:-3]
+
+        def dequote(m):
+            c = m.group(1)
+
+            if c == "{":
+                return "{{"
+            elif c == "[":
+                return "[["
+            elif c == "%":
+                return "%%"
+            elif c == "n":
+                return "\n"
+            elif c[0] == 'u':
+                group2 = m.group(2)
+
+                if group2:
+                    return unichr(int(m.group(2), 16))
+            else:
+                return c
+
+        if not raw:
+
+            # Collapse runs of whitespace into single spaces.
+            s = re.sub(r' *\n *', '\n', s)
+
+            rv = [ ]
+
+            for s in s.split(self.monologue_delimiter):
+                s = s.strip()
+
+                if not s:
+                    continue
+
+                s = re.sub(r'\s+', ' ', s)
+                s = re.sub(r'\\(u([0-9a-fA-F]{1,4})|.)', dequote, s)
+
+                rv.append(s)
+
+            return rv
+
+        return s
+
     def integer(self):
         """
         Tries to parse an integer. Returns a string containing the
@@ -822,6 +893,11 @@ class Lexer(object):
         oldpos = self.pos
         rv = self.word()
 
+        if (rv == "r") or (rv == "u"):
+            if self.text[self.pos:self.pos+1] in ( '"', "'", "`"):
+                self.pos = oldpos
+                return None
+
         if rv in KEYWORDS:
             self.pos = oldpos
             return None
@@ -892,6 +968,11 @@ class Lexer(object):
         oldpos = self.pos
         rv = self.match(image_word_regexp)
 
+        if (rv == "r") or (rv == "u"):
+            if self.text[self.pos:self.pos+1] in ( '"', "'", "`"):
+                self.pos = oldpos
+                return None
+
         if rv in KEYWORDS:
             self.pos = oldpos
             return None
@@ -1493,7 +1574,7 @@ def parse_menu(stmtl, loc):
     if has_say:
         rv.append(ast.Say(loc, say_who, say_what, None, interact=False))
 
-    rv.append(ast.Menu(loc, items, set, with_))
+    rv.append(ast.Menu(loc, items, set, with_, has_say or has_caption))
 
     return rv
 
@@ -2172,6 +2253,23 @@ def init_statement(l, loc):
     return ast.Init(loc, block, priority + l.init_offset)
 
 
+@statement("rpy monologue")
+def rpy_statement(l, loc):
+
+    if l.keyword("double"):
+        l.monologue_delimiter = "\n\n"
+    elif l.keyword("single"):
+        l.monologue_delimiter = "\n"
+    else:
+        l.error("rpy monologue expects either single or double.")
+
+    l.expect_eol()
+    l.expect_noblock('rpy monologue')
+    l.advance()
+
+    return [ ]
+
+
 def screen1_statement(l, loc):
 
     # The guts of screen language parsing is in screenlang.py. It
@@ -2277,7 +2375,11 @@ def translate_strings(init_loc, language, l):
                 ll.error("previous string is missing a translation")
 
             loc = ll.get_location()
-            old = parse_string(ll.rest())
+
+            try:
+                old = parse_string(ll.rest())
+            except:
+                ll.error("Could not parse string.")
 
         elif ll.keyword('new'):
 
@@ -2285,7 +2387,10 @@ def translate_strings(init_loc, language, l):
                 ll.error('no string to translate')
 
             newloc = ll.get_location()
-            new = parse_string(ll.rest())
+            try:
+                new = parse_string(ll.rest())
+            except:
+                ll.error("Could not parse string.")
 
             block.append(renpy.ast.TranslateString(loc, language, old, new, newloc))
 
@@ -2474,7 +2579,21 @@ def finish_say(l, loc, who, what, attributes=None):
 
             arguments = args
 
-    return ast.Say(loc, who, what, with_, attributes=attributes, interact=interact, arguments=arguments)
+    if isinstance(what, list):
+
+        rv = [ ]
+
+        for i in what:
+
+            if i == "{clear}":
+                rv.append(ast.UserStatement(loc, "nvl clear", [ ]))
+            else:
+                rv.append(ast.Say(loc, who, i, with_, attributes=attributes, interact=interact, arguments=arguments))
+
+        return rv
+
+    else:
+        return ast.Say(loc, who, what, with_, attributes=attributes, interact=interact, arguments=arguments)
 
 
 @statement("")
@@ -2483,7 +2602,7 @@ def say_statement(l, loc):
     state = l.checkpoint()
 
     # Try for a single-argument say statement.
-    what = l.string()
+    what = l.triple_string() or l.string()
 
     rv = finish_say(l, loc, None, what)
 
@@ -2518,7 +2637,7 @@ def say_statement(l, loc):
     else:
         attributes = None
 
-    what = l.string()
+    what = l.triple_string() or l.string()
 
     if (who is not None) and (what is not None):
 
@@ -2650,11 +2769,15 @@ def report_parse_errors():
         except:
             pass
 
-        print()
         print(file=f)
-        print(i)
         print(i, file=f)
 
+        try:
+            print()
+            print(i)
+        except:
+            pass
+
     print(file=f)
     print("Ren'Py Version:", renpy.version, file=f)
     print(time.ctime(), file=f)
