@@ -19,6 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
+
 import renpy.display
 import contextlib
 
@@ -80,9 +82,6 @@ class Positional(object):
         if parser:
             parser.add(self)
 
-# Used to generate the documentation
-all_keyword_names = set()
-
 
 class Keyword(object):
     """
@@ -92,11 +91,10 @@ class Keyword(object):
     def __init__(self, name):
         self.name = name
 
-        all_keyword_names.add(self.name)
-
         if parser:
             parser.add(self)
 
+
 STYLE_PREFIXES = [
     '',
     'insensitive_',
@@ -119,9 +117,6 @@ class Style(object):
     def __init__(self, name):
         self.name = name
 
-        for j in STYLE_PREFIXES:
-            all_keyword_names.add(j + self.name)
-
         if parser:
             parser.add(self)
 
@@ -135,9 +130,6 @@ class PrefixStyle(object):
         self.prefix = prefix
         self.name = name
 
-        for j in STYLE_PREFIXES:
-            all_keyword_names.add(prefix + j + self.name)
-
         if parser:
             parser.add(self)
 
@@ -625,6 +617,7 @@ bar_properties = [ Style(i) for i in [
 box_properties = [ Style(i) for i in [
     "box_layout",
     "box_wrap",
+    "box_wrap_spacing",
     "box_reverse",
     "order_reverse",
     "spacing",
@@ -889,6 +882,7 @@ Keyword("droppable")
 Keyword("drag_raise")
 Keyword("dragged")
 Keyword("dropped")
+Keyword("drop_allowable")
 Keyword("drag_handle")
 Keyword("drag_joined")
 Keyword("clicked")
@@ -900,6 +894,7 @@ add(ui_properties)
 add(position_properties)
 
 FunctionStatementParser("draggroup", "ui.draggroup", many)
+Keyword("min_overlap")
 add(ui_properties)
 add(position_properties)
 
@@ -922,6 +917,7 @@ class PassParser(Parser):
     def parse(self, l, name):
         return self.parse_exec("pass", l.number)
 
+
 PassParser("pass")
 
 
@@ -940,6 +936,7 @@ class DefaultParser(Parser):
 
         return self.parse_exec(code, l.number)
 
+
 DefaultParser("default")
 
 
@@ -981,6 +978,7 @@ class UseParser(Parser):
 
         return self.parse_exec(code, lineno)
 
+
 UseParser("use")
 
 
@@ -1044,6 +1042,7 @@ class IfParser(Parser):
 
         return [ rv ]
 
+
 IfParser("if")
 
 
@@ -1118,6 +1117,7 @@ class ForParser(Parser):
 
         return rv
 
+
 ForParser("for")
 
 
@@ -1143,6 +1143,7 @@ class PythonParser(Parser):
 
         return self.parse_exec(python_code, lineno)
 
+
 PythonParser("$", True)
 PythonParser("python", False)
 
@@ -1358,6 +1359,7 @@ class ScreenParser(Parser):
 
         return screen
 
+
 screen_parser = ScreenParser()
 screen_parser.add(all_statements)
 
