@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,30 +26,34 @@
 # When adding fields to a class in an __init__ method, we need to ensure that
 # field is copied in the copy() method.
 
+from __future__ import print_function
 
 import ast
 import collections
 import linecache
 from cPickle import loads, dumps
 import zlib
+import weakref
 
 import renpy.display
 import renpy.pyanalysis
 import renpy.sl2
 
-from renpy.display.motion import Transform
+from renpy.display.transform import Transform, ATLTransform
 from renpy.display.layout import Fixed
 from renpy.display.predict import displayable as predict_displayable
 
 from renpy.python import py_eval_bytecode
 from renpy.pyanalysis import Analysis, NOT_CONST, GLOBAL_CONST, ccache
-import md5
+
+import hashlib
+import time
 
 # This file contains the abstract syntax tree for a screen language
 # screen.
 
 # A serial number that makes each SLNode unique.
-serial = 0
+serial = int(time.time() * 1000000)
 
 # A sentinel used to indicate we should use the value found in the
 # expression.
@@ -61,6 +65,7 @@ filename = '<screen language>'
 # A log that's used for profiling information.
 profile_log = renpy.log.open("profile_screen", developer=True, append=False, flush=False)
 
+
 def compile_expr(node):
     """
     Wraps the node in a python AST, and compiles it.
@@ -96,11 +101,19 @@ class SLContext(renpy.ui.Addable):
         self.keywords = { }
 
         # The style prefix that is given to children of this displayable.
-        self.style_prefix = ""
+        self.style_prefix = None
 
         # A cache associated with this context. The cache maps from
         # statement serial to information associated with the statement.
-        self.cache = { }
+        self.new_cache = { }
+
+        # The old cache, used to take information from the old version of
+        # this displayable.
+        self.old_cache = { }
+
+        # The miss cache, used to take information that isn't present in
+        # old_cache.
+        self.miss_cache = { }
 
         # The number of times a particular use statement has been called
         # in the current screen. We use this to generate a unique name for
@@ -147,16 +160,9 @@ class SLContext(renpy.ui.Addable):
         # to speed things up.
         self.unlikely = False
 
-
-    def get_style_group(self):
-        style_prefix = self.style_prefix
-
-        if style_prefix:
-            return style_prefix[:-1]
-        else:
-            return None
-
-    style_group = property(get_style_group)
+        # The old and new generations of the use_cache.
+        self.new_use_cache = { }
+        self.old_use_cache = { }
 
     def add(self, d, key):
         self.children.append(d)
@@ -214,7 +220,6 @@ class SLNode(object):
 
         raise Exception("copy not implemented by " + type(self).__name__)
 
-
     def report_traceback(self, name, last):
         if last:
             return None
@@ -296,15 +301,20 @@ class SLNode(object):
 
         return False
 
+
 # A sentinel used to indicate a keyword argument was not given.
 NotGiven = renpy.object.Sentinel("NotGiven")
 
+
 class SLBlock(SLNode):
     """
     Represents a screen language block that can contain keyword arguments
     and child displayables.
     """
 
+    # RawBlock from parse or None if not present.
+    atl_transform = None
+
     def __init__(self, loc):
         SLNode.__init__(self, loc)
 
@@ -351,7 +361,7 @@ class SLBlock(SLNode):
                 keyword_values[k] = py_eval_bytecode(compile_expr(node))
             else:
                 keyword_keys.append(ast.Str(s=k))
-                keyword_exprs.append(node) # Will be compiled as part of ast.Dict below.
+                keyword_exprs.append(node)  # Will be compiled as part of ast.Dict below.
 
             self.constant = min(self.constant, const)
 
@@ -370,6 +380,13 @@ class SLBlock(SLNode):
         self.has_keyword = bool(self.keyword)
         self.keyword_children = [ ]
 
+        if self.atl_transform is not None:
+            self.has_keyword = True
+
+            self.atl_transform.mark_constant()
+            const = self.atl_transform.constant
+            self.constant = min(self.constant, const)
+
         for i in self.children:
             if i.has_keyword:
                 self.keyword_children.append(i)
@@ -379,7 +396,6 @@ class SLBlock(SLNode):
                 self.last_keyword = True
                 break
 
-
     def execute(self, context):
 
         # Note: SLBlock.execute() is inlined in various locations for performance
@@ -408,12 +424,17 @@ class SLBlock(SLNode):
         for i in self.keyword_children:
             i.keywords(context)
 
-        style_group = context.keywords.pop("style_group", NotGiven)
-        if style_group is not NotGiven:
-            if style_group is not None:
-                context.style_prefix = style_group + "_"
-            else:
-                context.style_prefix = ""
+        if self.atl_transform is not None:
+            transform = ATLTransform(self.atl_transform, context=context.scope)
+            context.keywords["at"] = transform
+
+        style_prefix = context.keywords.pop("style_prefix", NotGiven)
+
+        if style_prefix is NotGiven:
+            style_prefix = context.keywords.pop("style_group", NotGiven)
+
+        if style_prefix is not NotGiven:
+            context.style_prefix = style_prefix
 
     def copy_on_change(self, cache):
         for i in self.children:
@@ -433,6 +454,7 @@ class SLBlock(SLNode):
 
 list_or_tuple = (list, tuple)
 
+
 class SLCache(object):
     """
     The type of cache associated with an SLDisplayable.
@@ -485,10 +507,12 @@ class SLCache(object):
         # The style prefix used when this statement was first created.
         self.style_prefix = None
 
+
 # A magic value that, if returned by a displayable function, is not added to
 # the parent.
 NO_DISPLAYABLE = renpy.display.layout.Null()
 
+
 class SLDisplayable(SLBlock):
     """
     A screen language AST node that corresponds to a displayable being
@@ -497,6 +521,9 @@ class SLDisplayable(SLBlock):
 
     hotspot = False
 
+    # A list of variables that are locally constant.
+    local_constant = [ ]
+
     def __init__(self, loc, displayable, scope=False, child_or_fixed=False, style=None, text_style=None, pass_context=False, imagemap=False, replaces=False, default_keywords={}, hotspot=False):
         """
         `displayable`
@@ -584,6 +611,11 @@ class SLDisplayable(SLBlock):
         if self.imagemap:
             analysis.pop_control()
 
+        # If we use a scope, store the local constants that need to be
+        # kept and placed into the scope.
+        if self.scope:
+            self.local_constant = list(analysis.local_constant)
+
     def prepare(self, analysis):
 
         SLBlock.prepare(self, analysis)
@@ -606,7 +638,7 @@ class SLDisplayable(SLBlock):
                 has_values = True
             else:
                 values.append(use_expression)
-                exprs.append(node) # Will be compiled as part of the tuple.
+                exprs.append(node)  # Will be compiled as part of the tuple.
                 has_exprs = True
 
             self.constant = min(self.constant, const)
@@ -646,10 +678,12 @@ class SLDisplayable(SLBlock):
 
         screen = renpy.ui.screen
 
-        cache = context.cache.get(self.serial, None)
+        cache = context.old_cache.get(self.serial, None) or context.miss_cache.get(self.serial, None)
 
-        if cache is None:
-            context.cache[self.serial] = cache = SLCache()
+        if not isinstance(cache, SLCache):
+            cache = SLCache()
+
+        context.new_cache[self.serial] = cache
 
         copy_on_change = cache.copy_on_change
 
@@ -658,13 +692,20 @@ class SLDisplayable(SLBlock):
 
         if cache.constant and (cache.style_prefix == context.style_prefix):
 
-            for i in cache.constant_uses_scope:
+            for i, local_scope in cache.constant_uses_scope:
+
+                if local_scope:
+                    scope = dict(context.scope)
+                    scope.update(local_scope)
+                else:
+                    scope = context.scope
+
                 if copy_on_change:
-                    if i._scope(context.scope, False):
+                    if i._scope(scope, False):
                         cache.constant = None
                         break
                 else:
-                    i._scope(context.scope, True)
+                    i._scope(scope, True)
 
             else:
 
@@ -725,6 +766,7 @@ class SLDisplayable(SLBlock):
 
             arguments = keywords.pop("arguments", None)
             properties = keywords.pop("properties", None)
+            style_suffix = keywords.pop("style_suffix", None) or self.style
 
             if arguments:
                 positional += arguments
@@ -733,8 +775,11 @@ class SLDisplayable(SLBlock):
                 keywords.update(properties)
 
             # If we don't know the style, figure it out.
-            if ("style" not in keywords) and self.style:
-                keywords["style"] = ctx.style_prefix + self.style
+            if ("style" not in keywords) and style_suffix:
+                if ctx.style_prefix is None:
+                    keywords["style"] = style_suffix
+                else:
+                    keywords["style"] = ctx.style_prefix + "_" + style_suffix
 
             if widget_id and (widget_id in screen.widget_properties):
                 keywords.update(screen.widget_properties[widget_id])
@@ -794,7 +839,7 @@ class SLDisplayable(SLBlock):
                 if self.scope:
                     keywords["scope"] = ctx.scope
 
-                if self.replaces and context.updating:
+                if self.replaces and ctx.updating:
                     keywords['replaces'] = old_main
 
                 # Pass the context
@@ -812,8 +857,8 @@ class SLDisplayable(SLBlock):
 
                 imagemap = self.imagemap
 
-                cache.copy_on_change = False # We no longer need to copy on change.
-                cache.children = None # Re-add the children.
+                cache.copy_on_change = False  # We no longer need to copy on change.
+                cache.children = None  # Re-add the children.
 
             if debug:
                 if reused:
@@ -847,6 +892,8 @@ class SLDisplayable(SLBlock):
 
         finally:
 
+            ctx.keywords = None
+
             stack.pop()
 
             if imagemap:
@@ -921,20 +968,31 @@ class SLDisplayable(SLBlock):
 
         if (transform is not None) and (d is not NO_DISPLAYABLE):
             if reused and (transform == cache.raw_transform):
+
+                if isinstance(cache.transform, renpy.display.transform.Transform):
+                    if cache.transform.child is not d:
+                        cache.transform.set_child(d, duplicate=False)
+
                 d = cache.transform
             else:
                 cache.raw_transform = transform
 
                 if isinstance(transform, Transform):
                     d = transform(child=d)
+                    d._unique()
+
                 elif isinstance(transform, list_or_tuple):
                     for t in transform:
                         if isinstance(t, Transform):
                             d = t(child=d)
                         else:
                             d = t(d)
+
+                        d._unique()
+
                 else:
                     d = transform(d)
+                    d._unique()
 
                 if isinstance(d, Transform):
                     old_transform = cache.transform
@@ -959,7 +1017,14 @@ class SLDisplayable(SLBlock):
                 cache.constant = d
 
                 if self.scope and main._uses_scope:
-                    ctx.uses_scope.append(main)
+
+                    local_scope = { }
+
+                    for i in self.local_constant:
+                        if i in ctx.scope:
+                            local_scope[i] = ctx.scope[i]
+
+                    ctx.uses_scope.append((main, local_scope))
 
                 cache.constant_uses_scope = ctx.uses_scope
 
@@ -1055,7 +1120,7 @@ class SLDisplayable(SLBlock):
     def copy_on_change(self, cache):
         c = cache.get(self.serial, None)
 
-        if c is not None:
+        if isinstance(c, SLCache):
             c.copy_on_change = True
 
         for i in self.children:
@@ -1147,7 +1212,6 @@ class SLIf(SLNode):
             context.predicted.add(self.serial)
 
         for cond, block in self.prepared_entries:
-
             try:
                 cond_value = (cond is None) or eval(cond, context.globals, context.scope)
             except:
@@ -1201,6 +1265,7 @@ class SLIf(SLNode):
 
         return False
 
+
 class SLShowIf(SLNode):
     """
     The AST node that corresponds to the showif statement.
@@ -1250,7 +1315,10 @@ class SLShowIf(SLNode):
 
     def execute(self, context):
 
-        first_true = True
+        # This is true when the block should be executed - when no outer
+        # showif is False, and when no prior block in this showif has
+        # executed.
+        first_true = context.showif is not False
 
         for cond, block in self.prepared_entries:
 
@@ -1293,11 +1361,14 @@ class SLFor(SLBlock):
     simple for loops that assign a single variable.
     """
 
-    def __init__(self, loc, variable, expression):
+    index_expression = None
+
+    def __init__(self, loc, variable, expression, index_expression):
         SLBlock.__init__(self, loc)
 
         self.variable = variable
         self.expression = expression
+        self.index_expression = index_expression
 
     def copy(self, transclude):
         rv = self.instantiate(transclude)
@@ -1320,7 +1391,6 @@ class SLFor(SLBlock):
 
         analysis.pop_control()
 
-
     def prepare(self, analysis):
         node = ccache.ast_eval(self.expression)
 
@@ -1341,7 +1411,6 @@ class SLFor(SLBlock):
 
     def execute(self, context):
 
-
         variable = self.variable
         expr = self.expression_expr
 
@@ -1356,8 +1425,17 @@ class SLFor(SLBlock):
 
             value = [ 0 ]
 
-        newcaches = {}
-        oldcaches = context.cache.get(self.serial, newcaches)
+        newcaches = { }
+
+        oldcaches = context.old_cache.get(self.serial, newcaches) or { }
+
+        if not isinstance(oldcaches, dict):
+            oldcaches = { }
+
+        misscaches = context.miss_cache.get(self.serial, newcaches) or { }
+
+        if not isinstance(misscaches, dict):
+            misscaches = { }
 
         ctx = SLContext(context)
 
@@ -1365,13 +1443,20 @@ class SLFor(SLBlock):
 
             ctx.scope[variable] = v
 
-            cache = oldcaches.get(index, None)
+            if self.index_expression is not None:
+                index = eval(self.index_expression, ctx.globals, ctx.scope)
 
-            if cache is None:
-                cache = {}
+            ctx.old_cache = oldcaches.get(index, None) or { }
 
-            newcaches[index] = cache
-            ctx.cache = cache
+            if not isinstance(ctx.old_cache, dict):
+                ctx.old_cache = {}
+
+            ctx.miss_cache = misscaches.get(index, None) or { }
+
+            if not isinstance(ctx.miss_cache, dict):
+                ctx.miss_cache = {}
+
+            newcaches[index] = ctx.new_cache = { }
 
             # Inline of SLBlock.execute.
 
@@ -1385,7 +1470,7 @@ class SLFor(SLBlock):
             if context.unlikely:
                 break
 
-        context.cache[self.serial] = newcaches
+        context.new_cache[self.serial] = newcaches
 
         if ctx.fail:
             context.fail = True
@@ -1395,7 +1480,8 @@ class SLFor(SLBlock):
 
     def copy_on_change(self, cache):
         c = cache.get(self.serial, None)
-        if c is None:
+
+        if not isinstance(c, dict):
             return
 
         for child_cache in c.values():
@@ -1439,6 +1525,7 @@ class SLPass(SLNode):
 
         return rv
 
+
 class SLDefault(SLNode):
 
     def __init__(self, loc, variable, expression):
@@ -1543,7 +1630,12 @@ class SLUse(SLNode):
         target = renpy.display.screen.get_screen_variant(self.target)
 
         if target is None:
-            raise Exception("A screen named {} does not exist.".format(self.target))
+            self.constant = NOT_CONST
+
+            if renpy.config.developer:
+                raise Exception("A screen named {} does not exist.".format(self.target))
+            else:
+                return
 
         if target.ast is None:
             self.constant = NOT_CONST
@@ -1588,41 +1680,29 @@ class SLUse(SLNode):
 
         # Figure out the cache to use.
 
-        # True if we want to force-mark this as an update.
-        update = False
+        ctx = SLContext(context)
+        ctx.new_cache = context.new_cache[self.serial] = { }
+        ctx.miss_cache = context.miss_cache.get(self.serial, None) or { }
 
-        if (not context.predicting) and self.id:
+        if self.id:
 
-            # If we have an id, look it up in the current screen's use_cache.
-
-            current_screen = renpy.display.screen.current_screen()
             use_id = (self.target, eval(self.id, context.globals, context.scope))
 
-            cache = current_screen.use_cache.get(use_id, None)
+            ctx.old_cache = context.old_use_cache.get(use_id, None) or context.old_cache.get(self.serial, None) or { }
 
-            if cache is not None:
-                update = True
+            if use_id in ctx.old_use_cache:
+                ctx.updating = True
 
-            else:
-
-                if cache is None:
-                    cache = context.cache.get(self.serial, None)
-
-                if cache is None:
-                    cache = { }
-
-            context.cache[self.serial] = cache
-            current_screen.use_cache[use_id] = cache
+            ctx.new_use_cache[use_id] = ctx.new_cache
 
         else:
 
-            # Otherwise, look up the cache based on the statement's location.
-
-            cache = context.cache.get(self.serial, None)
-
-            if cache is None:
-                context.cache[self.serial] = cache = { }
+            ctx.old_cache = context.old_cache.get(self.serial, None) or { }
 
+        if not isinstance(ctx.old_cache, dict):
+            ctx.old_cache = { }
+        if not isinstance(ctx.miss_cache, dict):
+            ctx.miss_cache = { }
 
         # Evaluate the arguments.
         try:
@@ -1642,12 +1722,8 @@ class SLUse(SLNode):
         if ast.parameters is not None:
             new_scope = ast.parameters.apply(args, kwargs, ignore_errors=context.predicting)
 
-            scope = cache.get("scope", None)
-
-            if scope is None:
-                scope = cache["scope"] = new_scope
-            else:
-                scope.update(new_scope)
+            scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
+            scope.update(new_scope)
 
         else:
 
@@ -1658,19 +1734,18 @@ class SLUse(SLNode):
             scope.update(kwargs)
 
         scope["_scope"] = scope
+        ctx.new_cache["scope"] = scope
 
         # Run the child screen.
-        ctx = SLContext(context)
         ctx.scope = scope
-        ctx.cache = cache
-        ctx.parent = context
-
-        if update:
-            ctx.updating = True
+        ctx.parent = weakref.ref(context)
 
         ctx.transclude = self.block
 
-        ast.execute(ctx)
+        try:
+            ast.execute(ctx)
+        finally:
+            del scope["_scope"]
 
         if ctx.fail:
             context.fail = True
@@ -1693,7 +1768,6 @@ class SLTransclude(SLNode):
     def __init__(self, loc):
         SLNode.__init__(self, loc)
 
-
     def copy(self, transclude):
         rv = self.instantiate(transclude)
         rv.constant = transclude
@@ -1704,22 +1778,32 @@ class SLTransclude(SLNode):
         if not context.transclude:
             return
 
-        cache = context.cache.get(self.serial, None)
+        parent = context.parent
+        if parent is not None:
+            parent = parent()
 
-        if cache is None:
-            context.cache[self.serial] = cache = { }
+        ctx = SLContext(parent)
+        ctx.new_cache = context.new_cache[self.serial] = { }
+        ctx.old_cache = context.old_cache.get(self.serial, None) or { }
+        ctx.miss_cache = context.miss_cache.get(self.serial, None) or { }
 
-        cache["transclude"] = context.transclude
+        if not isinstance(ctx.old_cache, dict):
+            ctx.old_cache = { }
+        if not isinstance(ctx.miss_cache, dict):
+            ctx.miss_cache = { }
 
-        ctx = SLContext(context.parent)
-
-        ctx.cache = cache
+        ctx.new_cache["transclude"] = context.transclude
 
         ctx.children = context.children
         ctx.showif = context.showif
         ctx.uses_scope = context.uses_scope
 
-        context.transclude.execute(ctx)
+        try:
+            renpy.ui.stack.append(ctx)
+            context.transclude.keywords(ctx)
+            context.transclude.execute(ctx)
+        finally:
+            renpy.ui.stack.pop()
 
         if ctx.fail:
             context.fail = True
@@ -1744,7 +1828,6 @@ class SLScreen(SLBlock):
 
     version = 0
 
-
     # This screen's AST when the transcluded block is entirely
     # constant (or there is no transcluded block at all). This may be
     # the actual AST, or a copy.
@@ -1757,6 +1840,7 @@ class SLScreen(SLBlock):
     # The analysis
     analysis = None
 
+    layer = "'screens'"
 
     def __init__(self, loc):
 
@@ -1775,10 +1859,10 @@ class SLScreen(SLBlock):
         self.tag = None
 
         # The variant of screen we're defining.
-        self.variant = "None" # expr.
+        self.variant = "None"  # expr.
 
         # Should we predict this screen?
-        self.predict = "None" # expr.
+        self.predict = "None"  # expr.
 
         # The parameters this screen takes.
         self.parameters = None
@@ -1822,6 +1906,7 @@ class SLScreen(SLBlock):
             predict=renpy.python.py_eval(self.predict),
             parameters=self.parameters,
             location=self.location,
+            layer=renpy.python.py_eval(self.layer),
             )
 
     def analyze(self, analysis):
@@ -1850,7 +1935,6 @@ class SLScreen(SLBlock):
             self.not_const_ast = self.const_ast
             targets = [ self.const_ast ]
 
-
         for ast in targets:
             analysis = ast.analysis = Analysis(None)
 
@@ -1932,7 +2016,11 @@ class SLScreen(SLBlock):
 
         current_screen = renpy.display.screen.current_screen()
 
+        if current_screen.screen_name[0] in renpy.config.profile_screens:
+            debug = True
+
         context = SLContext()
+
         context.scope = scope
         context.globals = renpy.python.store_dicts["store"]
         context.debug = debug
@@ -1940,20 +2028,32 @@ class SLScreen(SLBlock):
         context.updating = (current_screen.phase == renpy.display.screen.UPDATE)
 
         name = scope["_name"]
-        main_cache = current_screen.cache
 
-        cache = main_cache.get(name, None)
-        if cache is None or (cache["version"] != self.version):
-            cache = { "version" : self.version }
-            main_cache[name] = cache
+        def get_cache(d):
+            rv = d.get(name, None)
 
-        context.cache = cache
+            if (not isinstance(rv, dict)) or (rv.get("version", None) != self.version):
+                rv = { "version" : self.version }
+                d[name] = rv
+
+            return rv
+
+        context.old_cache = get_cache(current_screen.cache)
+        context.miss_cache = get_cache(current_screen.miss_cache)
+        context.new_cache = { "version" : self.version }
+
+        context.old_use_cache = current_screen.use_cache
+        context.new_use_cache = { }
 
         self.const_ast.execute(context)
 
         for i in context.children:
             renpy.ui.implicit_add(i)
 
+        current_screen.cache[name] = context.new_cache
+        current_screen.use_cache = context.new_use_cache
+
+
 class ScreenCache(object):
 
     def __init__(self):
@@ -1964,18 +2064,20 @@ class ScreenCache(object):
 
         self.updated = False
 
+
 scache = ScreenCache()
 
 CACHE_FILENAME = "cache/screens.rpyb"
 
+
 def load_cache():
-    if renpy.game.args.compile: # @UndefinedVariable
+    if renpy.game.args.compile:  # @UndefinedVariable
         return
 
     try:
         f = renpy.loader.load(CACHE_FILENAME)
 
-        digest = f.read(md5.digest_size)
+        digest = f.read(hashlib.md5().digest_size)
         if digest != renpy.game.script.digest.digest():
             return
 
@@ -1990,10 +2092,14 @@ def load_cache():
     except:
         pass
 
+
 def save_cache():
     if not scache.updated:
         return
 
+    if renpy.macapp:
+        return
+
     try:
         data = zlib.compress(dumps(scache, 2), 9)
 
@@ -2002,5 +2108,3 @@ def save_cache():
             f.write(data)
     except:
         pass
-
-
