@@ -90,7 +90,7 @@ class ShowIf(renpy.display.layout.Container):
         if self.show_child:
             cw, ch = cr.get_size()
             rv = renpy.display.render.Render(cw, ch)
-            rv.blit(cr, (0, 0))
+            rv.blit(cr, (0, 0), focus=self.condition)
         else:
             rv = renpy.display.render.Render(0, 0)
 
@@ -211,6 +211,7 @@ add(text_text_properties)
 DisplayableParser("label", renpy.ui._label, "label", 0, scope=True)
 Positional("label")
 Keyword("text_style")
+Keyword("substitute")
 add(window_properties)
 add(text_position_properties)
 add(text_text_properties)
@@ -347,6 +348,7 @@ Keyword("xinitial")
 Keyword("yinitial")
 Keyword("scrollbars")
 Keyword("spacing")
+Keyword("transpose")
 Style("xminimum")
 Style("yminimum")
 PrefixStyle("side_", "spacing")
@@ -373,6 +375,7 @@ Keyword("xinitial")
 Keyword("yinitial")
 Keyword("scrollbars")
 Keyword("spacing")
+Keyword("transpose")
 Style("spacing")
 Style("xminimum")
 Style("yminimum")
@@ -451,12 +454,14 @@ for name in [ "add", "image" ]:
         Style(i)
 
 DisplayableParser("drag", renpy.display.dragdrop.Drag, "drag", 1, replaces=True)
+Keyword("activated")
 Keyword("drag_name")
 Keyword("draggable")
 Keyword("droppable")
 Keyword("drag_raise")
 Keyword("dragged")
 Keyword("dropped")
+Keyword("drop_allowable")
 Keyword("drag_handle")
 Keyword("drag_joined")
 Keyword("drag_offscreen")
@@ -465,9 +470,11 @@ Keyword("hovered")
 Keyword("unhovered")
 Keyword("focus_mask")
 Keyword("mouse_drop")
+Keyword("alternate")
 Style("child")
 
 DisplayableParser("draggroup", renpy.display.dragdrop.DragGroup, None, many, replaces=True)
+Keyword("min_overlap")
 
 DisplayableParser("mousearea", renpy.display.behavior.MouseArea, 0, replaces=True)
 Keyword("hovered")
