@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,6 +21,8 @@
 
 # This module contains code to support user-defined statements.
 
+from __future__ import print_function
+
 import renpy
 
 # The statement registry. It's a map from tuples giving the prefixes of
@@ -30,7 +32,22 @@ registry = { }
 parsers = renpy.parser.ParseTrie()
 
 
-def register(name, parse=None, lint=None, execute=None, predict=None, next=None, scry=None, block=False, init=False, translatable=False):  # @ReservedAssignment
+def register(
+        name,
+        parse=None,
+        lint=None,
+        execute=None,
+        predict=None,
+        next=None,
+        scry=None,
+        block=False,
+        init=False,
+        translatable=False,
+        execute_init=None,
+        init_priority=0,
+        label=None,
+        warp=None,
+):
     """
     :doc: statement_register
     :name: renpy.register_statement
@@ -42,48 +59,85 @@ def register(name, parse=None, lint=None, execute=None, predict=None, next=None,
         empty string to define a new default statement (the default statement will
         replace the say statement).
 
+    `block`
+        When this is False, the statement does not expect a block. When True, it
+        expects a block, but leaves it up to the lexer to parse that block. If the
+        string "script", the block is interpreted as containing one or more
+        Ren'Py script language statements. If the string "possible", the
+        block expect condition is determined by the parse function.
+
     `parse`
         This is a function that takes a Lexer object. This function should parse the
         statement, and return an object. This object is passed as an argument to all the
         other functions. The lexer argument has the following methods:
 
     `lint`
-        This is called to check the statement. It is passed a single object, the
-        argument returned from parse. It should call renpy.error to report errors.
+        This is called to check the statement. It is passed a single argument, the
+        object returned from parse. It should call renpy.error to report errors.
 
     `execute`
         This is a function that is called when the statement executes. It is passed a
-        single object, the argument returned from parse.
+        single argument, the object returned from parse.
+
+    `execute_init`
+        This is a function that is called at init time, at priority 0.
 
     `predict`
         This is a function that is called to predict the images used by the statement.
-        It is passed a single object, the argument returned from parse. It should return
+        It is passed a single argument, the object returned from parse. It should return
         a list of displayables used by the statement.
 
     `next`
-        This is called to determine the next statement. It is passed a single object,
-        the argument returned from parse. It should either return a label, or return
-        None if execution should continue to the next statement.
+        This is a function that is called to determine the next statement.
+
+        If `block` is not "script", this is passed a single argument, the object
+        returned from the parse function. If `block` is "script", an additional
+        argument is passed, an object that names the first statement in the block.
+
+        The function should return either a string giving a label to jump to,
+        the second argument to transfer control into the block, or None to
+        continue to the statement after this one.
+
+    `label`
+        This is a function that is called to determine the label of this
+        statement. If it returns a string, that string is used as the statement
+        label, which can be called and jumped to like any other label.
+
+    `warp`
+        This is a function that is called to determine if this statement
+        should execute during warping. If the function exists and returns
+        true, it's run during warp, otherwise the statement is not run
+        during warp.
 
     `scry`
         Used internally by Ren'Py.
 
-    `block`
-        True if this takes a block, false otherwise.
-
     `init`
         True if this statement should be run at init-time. (If the statement
         is not already inside an init block, it's automatically placed inside
-        an init 0 block.)
+        an init block.) This calls the execute function, in addition to the
+        execute_init function.
+
+    `init_priority`
+        An integer that determines the priority of initialisation of init block.
+
     """
     name = tuple(name.split())
 
-    registry[name] = dict(parse=parse,
-                          lint=lint,
-                          execute=execute,
-                          predict=predict,
-                          next=next,
-                          scry=scry)
+    registry[name] = dict(
+        parse=parse,
+        lint=lint,
+        execute=execute,
+        execute_init=execute_init,
+        predict=predict,
+        next=next,
+        scry=scry,
+        label=label,
+        warp=warp,
+    )
+
+    if block not in [True, False, "script", "possible"]:
+        raise Exception("Unknown \"block\" argument value: {}".format(block))
 
     # The function that is called to create an ast.UserStatement.
     def parse_user_statement(l, loc):
@@ -93,17 +147,21 @@ def register(name, parse=None, lint=None, execute=None, predict=None, next=None,
             rv = renpy.ast.UserStatement(loc, l.text, l.subblock)
             rv.translatable = translatable
 
-            if not block:
+            if block is False:
                 l.expect_noblock(" ".join(name) + " statement")
-                l.advance()
-            else:
+            elif block is True:
                 l.expect_block(" ".join(name) + " statement")
-                l.advance()
+            elif block == "script":
+                l.expect_block(" ".join(name) + " statement")
+                rv.code_block = renpy.parser.parse_block(l.subblock_lexer())
+
+            l.advance()
+
         finally:
             renpy.exports.pop_error_handler()
 
         if init and not l.init:
-            rv = renpy.ast.Init(loc, [ rv ], 0)
+            rv = renpy.ast.Init(loc, [rv], init_priority + l.init_offset)
 
         return rv
 
